<?php

/**
 * @package    VP Email Manager
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 3 or later; see LICENSE.txt
 * @author     Abhishek Das <info@virtueplanet.com>
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 * @phpcs:disable PSR2.Classes.PropertyDeclaration.Underscore
 * @phpcs:disable PSR2.Methods.MethodDeclaration.Underscore
 * @phpcs:disable PSR1.Methods.CamelCapsMethodName
 */

use Joomla\CMS\Factory;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

class VpemHelperMustache
{
    protected $isHTML = false;

    protected $_mustache = null;

    protected $_constructor = null;

    protected $_debug = false;

    protected $properties = array();

    public function __construct($data = null, $mustache = null, $constructor = null)
    {
        $config = Factory::getConfig();

        if (($config->get('error_reporting') == 'development' || $config->get('error_reporting') == 'maximum') || $config->get('debug')) {
            $this->_debug = true;
        } else {
            $this->_debug = false;
        }

        if ($data) {
            if ($mustache instanceof Mustache_Engine) {
                $this->_mustache = $mustache;
            }

            if ($constructor) {
                $this->_constructor = $constructor;
            }

            // Over load data
            $this->setProperties($data);
        }
    }

    public function __isset($name)
    {
        if (array_key_exists($name, $this->properties)) {
            return true;
        }

        if (method_exists($this, '_' . $name)) {
            return true;
        }

        JPluginHelper::importPlugin('vpem');

        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            $results = Factory::getApplication()->triggerEvent('onIssetMethod', array($this, $name, get_class($this)));
        } else {
            /** @disregard P1009 */
            $results = JEventDispatcher::getInstance()->trigger('onIssetMethod', array($this, $name, get_class($this)));
        }

        foreach ($results as $result) {
            if ($result === true) {
                return true;
            }
        }

        return false;
    }

    /**
     * __get
     *
     * @param  mixed $name
     * @return void
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->properties)) {
            return $this->properties[$name];
        }

        if (method_exists($this, '_' . $name)) {
            return array($this, '_' . $name);
        }

        JPluginHelper::importPlugin('vpem');
        $customMethods = array();

        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            Factory::getApplication()->triggerEvent('onGetMethod', array($this, $name, get_class($this), &$customMethods));
        } else {
            /** @disregard P1009 */
            JEventDispatcher::getInstance()->trigger('onGetMethod', array($this, $name, get_class($this), &$customMethods));
        }

        foreach ($customMethods as $customMethod) {
            return $customMethod;
        }
    }

    public function _escape($string, \Mustache_LambdaHelper $helper)
    {
        $string = $helper->render($string);

        return htmlspecialchars($string, ENT_COMPAT, 'UTF-8');
    }

    public function _strip_tags($string, \Mustache_LambdaHelper $helper)
    {
        $string = $helper->render($string);

        return strip_tags($string);
    }

    public function _htmlspecialchars($string, \Mustache_LambdaHelper $helper)
    {
        if (strpos($string, ',') === false) {
            $args = array($string);
        } else {
            $args = explode(',', $string);
            $args = array_map('trim', $args);
        }

        $args[0] = $helper->render($args[0]);

        if (isset($args[1])) {
            $args[1] = constant($args[1]);
        }

        return call_user_func_array('htmlspecialchars', $args);
    }

    public function _round($string, \Mustache_LambdaHelper $helper)
    {
        if (empty($string)) {
            return $string;
        }

        $input = $string;
        $type  = 'vendor';

        if (strpos($string, ',') !== false) {
            $args = explode(',', $string);
            $args = array_map('trim', $args);

            if (isset($args[0])) {
                $input = $args[0];
            }

            if (isset($args[1])) {
                $type = (string) $args[1];
            }
        }

        $value = $helper->render($input);

        if (!is_numeric($value) && $value == $input) {
            return $input;
        }

        $currency = $this->_constructor->getCurrency($type);

        if ($type == 'user') {
            $value = $currency->roundForDisplay($value, $this->user_currency_id);
        } else {
            $value = $currency->roundForDisplay($value);
        }

        return $value;
    }

    public function _math($string, \Mustache_LambdaHelper $helper)
    {
        $value = $helper->render($string);

        if (empty($value)) {
            return $value;
        }

        JLoader::register('EvalMath', JPATH_ADMINISTRATOR . '/components/com_vpem/libraries/evalmath/evalmath.class.php');

        $m = new EvalMath();

        if (!$this->_debug) {
            $m->suppress_errors = true;
        } else {
            $m->suppress_errors = false;
        }

        $result = $m->evaluate($value);

        return $result;
    }

    public function _date($string, \Mustache_LambdaHelper $helper)
    {
        $input  = '';
        $format = 'Y-m-d';
        $offset = 0;

        if (!empty($string)) {
            $input = trim($string);
        }

        // Expected format (date, format, offset)
        // Offset must be in hours

        if (strpos($string, ',') !== false) {
            $args = explode(',', $string);
            $args = array_map('trim', $args);

            if (isset($args[0])) {
                $input = $args[0];
            }

            if (isset($args[1])) {
                $format = (string) $args[1];
            }

            if (isset($args[2])) {
                $offset = (float) $args[2];
            }
        }

        if (!empty($input)) {
            $input = $helper->render($input);
        }

        if ($input == JText::_('COM_VIRTUEMART_DELDATE_INV')) {
            $input = $helper->render('{{ created_on }}');
        }

        if (empty($input) || !self::validateDate($input)) {
            $input = 'now';
        }

        // Get Joomla! Global Config
        $config = Factory::getConfig();

        // Get a date object based on UTC.
        $date = Factory::getDate($input, 'UTC');

        // Set the correct time zone based on the server configuration.
        $date->setTimeZone(new DateTimeZone($config->get('offset')));

        if ($offset) {
            if ($offset < 0) {
                $interval = new DateInterval('PT' . ($offset * -1) . 'H');

                $date->sub($interval);
            } else {
                $interval = new DateInterval('PT' . $offset . 'H');

                $date->add($interval);
            }
        }

        return $date->format($format, true);
    }

    public function setVendor()
    {
        $vendorModel = VmModel::getModel('vendor');

        if (!empty($this->virtuemart_vendor_id)) {
            $vendor = $vendorModel->getVendor($this->virtuemart_vendor_id);
        } else {
            $vendor = $vendorModel->getVendor();
        }

        $vendorModel->addImages($vendor, 1);

        if (!empty($vendor->images)) {
            $vendor->vendor_image = '<img src="' . JUri::root() . $vendor->images[0]->file_url . '" style="width: ' . $vendor->vendor_letter_header_imagesize . 'mm; max-width: 100% !important;" />';
        }

        $vendorFields = $vendorModel->getVendorAddressFields($vendor->virtuemart_vendor_id);

        foreach ($vendorFields['fields'] as $field) {
            if ($field['type'] == 'delimiter') {
                continue;
            }

            $name = $field['name'];

            $vendor->$name = $field['value'];
        }

        if (property_exists($vendor, 'virtuemart_country_id')) {
            $vendor->virtuemart_country_name = $vendor->virtuemart_country_id;
        }

        if (property_exists($vendor, 'virtuemart_state_id')) {
            $vendor->virtuemart_state_name = $vendor->virtuemart_state_id;
        }

        if (VmConfig::get('enable_content_plugin', 0)) {
            shopFunctionsF::triggerContentPlugin($vendor, 'vendor', 'vendor_store_desc');
            shopFunctionsF::triggerContentPlugin($vendor, 'vendor', 'vendor_terms_of_service');
            shopFunctionsF::triggerContentPlugin($vendor, 'vendor', 'vendor_legal_info');
        }

        $this->set('vendor', $vendor);
    }

    protected function renderMustacheVar($string)
    {
        if ($this->_mustache) {
            $string = $this->_mustache->render($string, $this);
        }

        return $string;
    }

    protected static function validateDate($date)
    {
        $db = Factory::getDbo();
        $d  = DateTime::createFromFormat($db->getDateFormat(), $date);

        return $d && $d->format($db->getDateFormat()) === $date;
    }

    protected function setProperties($data)
    {
        $array = is_object($data) ? get_object_vars($data) : $data;

        foreach ($array as $property => $value) {
            $this->set($property, $value);
        }
    }

    public function set($property, $value)
    {
        $property = (string) $property;

        $this->properties[$property] = $value;
    }
}
