<?php

/**
 * @package    VP Email Manager
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 3 or later; see LICENSE.txt
 * @author     Abhishek Das <info@virtueplanet.com>
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Mailer helper class.
 */
class VpemHelperRenderer extends JObject
{
    protected $db;

    protected static $moustacheRegistered = false;

    protected static $instance = null;

    /**
     * Constructor.
     *
     * @param   array  $config  An optional associative array of configuration settings.
     *
     * @since   1.0
     */
    public function __construct($config = [])
    {
        $this->db = isset($config['db']) ? $config['db'] : Factory::getDbo();

        // Register Mustache Autoloader class
        JLoader::register('Mustache_Autoloader', JPATH_ADMINISTRATOR . '/components/com_vpem/libraries/Mustache/Autoloader.php');
    }

    /**
     * Method to get the mailer helper instance.
     *
     * @param  array  $config  An optional associative array of configuration settings.
     *
     * @return object  VPAdvancedUserHelperMailer instance
     */
    public static function getInstance($config = [])
    {
        if (self::$instance === null) {
            self::$instance = new self($config);
        }

        return self::$instance;
    }

    /**
     * Method to render email with Mustache.
     *
     * @param  array    $data            Data array
     * @param  string   $emailSubject    Empty email subject string
     * @param  string   $emailBody       Empty email body string
     * @param  objecy   $mustache        Optional Mustache instance
     * @param  boolean  $html_mode       Optional HTML mode. Default is html.
     *
     * @return boolean  False in case of failure.
     */
    public function renderMail($data, &$emailSubject, &$emailBody, $mustache = null, $html_mode = true)
    {
        $filter = JFilterInput::getInstance();

        if (!$mustache) {
            // Get Mustache instance
            if (!$mustache = $this->getMustache($html_mode)) {
                $this->setError($this->getError());
                return false;
            }
        }

        // Render email subject
        $emailSubject = $mustache->render($emailSubject, $data);
        $emailSubject = $filter->clean($emailSubject);

        // Render email body
        $emailBody    = $mustache->render($emailBody, $data);
        $emailBody    = $this->processBody($emailBody);

        if (!$html_mode) {
            // Ensure plain text email body
            $emailBody = $filter->clean($emailBody);
        }

        return true;
    }

    /**
     * Method to get a new Mustache engine instance.
     *
     * @param boolean $html_mode True if html mode
     *
     * @return mixed (object/boolean) False in case of failure
     */
    public function getMustache($html_mode = true)
    {
        // If Mustache Autoloader class is not found throw an error
        if (!class_exists('Mustache_Autoloader')) {
            $this->setError('Mustache library not loaded.');

            return false;
        }

        // Register Mustache autoloader if not registered yet
        if (!self::$moustacheRegistered) {
            Mustache_Autoloader::register();

            self::$moustacheRegistered = true;
        }

        // Build a new Mustache engine instance
        $mustache = new Mustache_Engine([
            'escape' => function ($value) {
                return $value;
            },
            'charset' => 'UTF-8',
            'strict_callables' => false
        ]);

        // Register helper classes
        JLoader::register('VpemHelperMustacheJText', __DIR__ . '/mustache/jtext.php');
        JLoader::register('VpemHelperMustacheDo', __DIR__ . '/mustache/do.php');

        // Add Mustache JText helper
        $JTextHelper = new VpemHelperMustacheJText(['html' => $html_mode]);
        $mustache->addHelper('JText', $JTextHelper);

        // Add Mustache Do helper
        $DoHelper = new VpemHelperMustacheDo();
        $mustache->addHelper('do', $DoHelper);

        return $mustache;
    }

    /**
     * Process email body with absolute src links.
     *
     * @param  string  $buffer  The email body.
     *
     * @return  string
     */
    protected function processBody($buffer)
    {
        // Replace src links.
        $base = Uri::root();

        // Check for all unknown protocols (a protocol must contain at least one alphanumeric character followed by a ":").
        $protocols  = '[a-zA-Z0-9\-]+:';
        $attributes = array('href=', 'src=', 'poster=');

        foreach ($attributes as $attribute) {
            if (strpos($buffer, $attribute) !== false) {
                $regex  = '#\s' . $attribute . '"(?!/|' . $protocols . '|\#|\')([^"]*)"#m';
                $buffer = preg_replace($regex, ' ' . $attribute . '"' . $base . '$1"', $buffer);

                $this->checkBuffer($buffer);
            }
        }

        if (strpos($buffer, 'srcset=') !== false) {
            $regex = '#\s+srcset="([^"]+)"#m';

            $buffer = preg_replace_callback(
                $regex,
                function ($match) use ($base, $protocols) {
                    preg_match_all('#(?:[^\s]+)\s*(?:[\d\.]+[wx])?(?:\,\s*)?#i', $match[1], $matches);

                    foreach ($matches[0] as &$src) {
                        $src = preg_replace('#^(?!/|' . $protocols . '|\#|\')(.+)#', $base . '$1', $src);
                    }

                    return ' srcset="' . implode($matches[0]) . '"';
                },
                $buffer
            );

            $this->checkBuffer($buffer);
        }

        // Replace all unknown protocols in javascript window open events.
        if (strpos($buffer, 'window.open(') !== false) {
            $regex  = '#onclick="window.open\(\'(?!/|' . $protocols . '|\#)([^/]+[^\']*?\')#m';
            $buffer = preg_replace($regex, 'onclick="window.open(\'' . $base . '$1', $buffer);

            $this->checkBuffer($buffer);
        }

        // Replace all unknown protocols in onmouseover and onmouseout attributes.
        $attributes = array('onmouseover=', 'onmouseout=');

        foreach ($attributes as $attribute) {
            if (strpos($buffer, $attribute) !== false) {
                $regex  = '#' . $attribute . '"this.src=([\']+)(?!/|' . $protocols . '|\#|\')([^"]+)"#m';
                $buffer = preg_replace($regex, $attribute . '"this.src=$1' . $base . '$2"', $buffer);

                $this->checkBuffer($buffer);
            }
        }

        // Replace all unknown protocols in CSS background image.
        if (strpos($buffer, 'style=') !== false) {
            $regex_url  = '\s*url\s*\(([\'\"]|\&\#0?3[49];)?(?!/|\&\#0?3[49];|' . $protocols . '|\#)([^\)\'\"]+)([\'\"]|\&\#0?3[49];)?\)';
            $regex  = '#style=\s*([\'\"])(.*):' . $regex_url . '#m';
            $buffer = preg_replace($regex, 'style=$1$2: url($3' . $base . '$4$5)', $buffer);

            $this->checkBuffer($buffer);
        }

        // Replace all unknown protocols in OBJECT param tag.
        if (strpos($buffer, '<param') !== false) {
            // OBJECT <param name="xx", value="yy"> -- fix it only inside the <param> tag.
            $regex  = '#(<param\s+)name\s*=\s*"(movie|src|url)"[^>]\s*value\s*=\s*"(?!/|' . $protocols . '|\#|\')([^"]*)"#m';
            $buffer = preg_replace($regex, '$1name="$2" value="' . $base . '$3"', $buffer);

            $this->checkBuffer($buffer);

            // OBJECT <param value="xx", name="yy"> -- fix it only inside the <param> tag.
            $regex  = '#(<param\s+[^>]*)value\s*=\s*"(?!/|' . $protocols . '|\#|\')([^"]*)"\s*name\s*=\s*"(movie|src|url)"#m';
            $buffer = preg_replace($regex, '<param value="' . $base . '$2" name="$3"', $buffer);

            $this->checkBuffer($buffer);
        }

        // Replace all unknown protocols in OBJECT tag.
        if (strpos($buffer, '<object') !== false) {
            $regex  = '#(<object\s+[^>]*)data\s*=\s*"(?!/|' . $protocols . '|\#|\')([^"]*)"#m';
            $buffer = preg_replace($regex, '$1data="' . $base . '$2"', $buffer);

            $this->checkBuffer($buffer);
        }

        return $buffer;
    }

    /**
     * Check the buffer.
     *
     * @param   string  $buffer  Buffer to be checked.
     *
     * @return  void
     */
    protected function checkBuffer($buffer)
    {
        if ($buffer === null) {
            switch (preg_last_error()) {
                case PREG_BACKTRACK_LIMIT_ERROR:
                    $message = 'PHP regular expression limit reached (pcre.backtrack_limit)';
                    break;
                case PREG_RECURSION_LIMIT_ERROR:
                    $message = 'PHP regular expression limit reached (pcre.recursion_limit)';
                    break;
                case PREG_BAD_UTF8_ERROR:
                    $message = 'Bad UTF8 passed to PCRE function';
                    break;
                default:
                    $message = 'Unknown PCRE error calling PCRE function';
            }

            throw new RuntimeException($message);
        }
    }
}
