<?php
/**
 * @package    VP Email Manager
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 3 or later; see LICENSE.txt
 * @author     Abhishek Das <info@virtueplanet.com>
 * @link       https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;
use Joomla\Registry\Registry;
use Joomla\CMS\Factory;
use Joomla\CMS\Access\Access;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Table\Table;

/**
 * event Table class
 */
class VpemTableOutbox extends \Joomla\CMS\Table\Table
{
	/**
	 * Constructor
	 *
	 * @param   JDatabase  &$db  A database connector object
	 */
	public function __construct(&$db)
	{
		parent::__construct('#__vpem_outbox', 'id', $db);
	}

	/**
	 * Overloaded bind function to pre-process the params.
	 *
	 * @param   array  $array   Named array
	 * @param   mixed  $ignore  Optional array or list of parameters to ignore
	 *
	 * @return  null|string  null is operation was satisfactory, otherwise returns an error
	 *
	 * @see     JTable:bind
	 * @throws  Exception
	 */
	public function bind($array, $ignore = '')
	{
		$date  = Factory::getDate();
		$input = Factory::getApplication()->input;
		$task  = $input->getString('task', '');

		if ((empty($array['id']) || $array['id'] == 0) && empty($array['sent_on']))
		{
			$array['sent_on'] = $date->toSql();
		}
		
		if (isset($array['mail_to']) && is_array($array['mail_to']))
		{
			$array['mail_to'] = implode(',', $array['mail_to']);
		}
		
		if (!isset($array['cc']))
		{
			$array['cc'] = '';
		}
		
		if (!isset($array['bcc']))
		{
			$array['bcc'] = '';
		}
		
		$array['status'] = !empty($array['status']) ? 1 : 0;
		
		if (isset($array['attachments']) && is_array($array['attachments']))
		{
			foreach ($array['attachments'] as $key => $file)
			{
				$file = JPath::clean($file);
				$root = JPath::clean(JPATH_ROOT . DIRECTORY_SEPARATOR);
				
				if (strpos($file, $root) === 0)
				{
					$file = substr($file, strlen($root));
				}
				
				$array['attachments'][$key] = $file;
			}
		}
		
		$multiple_fields = ['attachments'];
		
		foreach ($multiple_fields as $multiple_field)
		{
			if (isset($array[$multiple_field]) && is_array($array[$multiple_field]))
			{
				$registry = new Registry;
				$registry->loadArray($array[$multiple_field]);

				$array[$multiple_field] = (string) $registry;
			}
			elseif (!isset($array[$multiple_field]) || is_null($array[$multiple_field]))
			{
				$array[$multiple_field] = '{}';
			}
		}
		
		return parent::bind($array, $ignore);
	}

	/**
	 * This function convert an array of JAccessRule objects into an rules array.
	 *
	 * @param   array  $jaccessrules  An array of JAccessRule objects.
	 *
	 * @return  array
	 */
	private function JAccessRulestoArray($jaccessrules)
	{
		$rules = array();

		foreach ($jaccessrules as $action => $jaccess)
		{
			$actions = array();

			if ($jaccess)
			{
				foreach ($jaccess->getData() as $group => $allow)
				{
					$actions[$group] = ((bool)$allow);
				}
			}

			$rules[$action] = $actions;
		}

		return $rules;
	}

	/**
	 * Overloaded check function
	 *
	 * @return bool
	 */
	public function check()
	{
		return parent::check();
	}

	/**
	 * Define a namespaced asset name for inclusion in the #__assets table
	 *
	 * @return string The asset name
	 *
	 * @see Table::_getAssetName
	 */
	protected function _getAssetName()
	{
		$k = $this->_tbl_key;

		return 'com_vpem.outbox.' . (int) $this->$k;
	}

	/**
	 * Returns the parent asset's id. If you have a tree structure, retrieve the parent's id using the external key field
	 *
	 * @param   JTable   $table  Table name
	 * @param   integer  $id     Id
	 *
	 * @see Table::_getAssetParentId
	 *
	 * @return mixed The id on success, false on failure.
	 */
	protected function _getAssetParentId(JTable $table = null, $id = null)
	{
		// We will retrieve the parent-asset from the Asset-table
		$assetParent = Table::getInstance('Asset');

		// Default: if no asset-parent can be found we take the global asset
		$assetParentId = $assetParent->getRootId();

		// The item has the component as asset-parent
		$assetParent->loadByName('com_vpem');

		// Return the found asset-parent-id
		if ($assetParent->id)
		{
			$assetParentId = $assetParent->id;
		}

		return $assetParentId;
	}

	/**
	 * Delete a record by id
	 *
	 * @param   mixed  $pk  Primary key value to delete. Optional
	 *
	 * @return bool
	 */
	public function delete($pk = null)
	{
		$this->load($pk);
		$result = parent::delete($pk);
		
		return $result;
	}
}
