<?php
/**
 * @package    VP Email Manager
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @author     Abhishek Das <info@virtueplanet.com>
 * @link       https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

JLoader::import('joomla.filesystem.file');
JLoader::import('joomla.filesystem.folder');

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseModel;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;

/**
 * VP Email Manager Package Script
 * 
 * @since   1.0
 */
class pkg_vpemInstallerScript
{
	/**
	 * Called on installation
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function install($adapter)
	{
		$app = Factory::getApplication();
		
		if ($this->enablePlugin())
		{
			$app->enqueueMessage('VP Email Manager Component package has been successfully installed in your site.');
		}
		
		$this->updateUserAndDateColumns();
		
		return true;
	}

	/**
	 * Called on update
	 *
	 * @param   JAdapterInstance  $adapter  The object responsible for running this script
	 *
	 * @return  boolean  True on success
	 */
	public function update($adapter)
	{
		$app = Factory::getApplication();
		
		$this->deleteOutdatedFolders();
		
		return true;
	}

	/**
	 * Called before any type of action
	 *
	 * @param     string              $route      Which action is happening (install|uninstall|discover_install)
	 * @param     jadapterinstance    $adapter    The object responsible for running this script
	 *
	 * @return    boolean                         True on success
	 */
	public function preflight($route, $adapter)
	{
		return true;
	}


	/**
	 * Called after any type of action
	 *
	 * @param     string              $route      Which action is happening (install|uninstall|discover_install)
	 * @param     jadapterinstance    $adapter    The object responsible for running this script
	 *
	 * @return    boolean                         True on success
	 */
	public function postflight($route, $adapter)
	{
		return true;
	}
	
	/**
	* Private method to enable all required plugins of the component
	* 
	* @return boolean
	*/
	private function enablePlugin()
	{
		$app       = Factory::getApplication();
		$plugin_id = $this->getPluginID();
		
		if (!$plugin_id)
		{
			$app->enqueueMessage('Please go to Plugin Manager and enable "System - VP Email Manager" plugins.');
			
			return false;
		}

		jimport('legacy.model.legacy');
		
		JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_plugins/models/', 'PluginsModel');
		
		$app   = Factory::getApplication();
		$model = JModelLegacy::getInstance('Plugin', 'PluginsModel');
		
		if (!$model)
		{
			$app->enqueueMessage('Please go to Plugin Manager and enable "System - VP Email Manager" plugins.');
			
			return false;
		}
		
		$hasError = false;
		
		if (!$model->publish($plugin_id, 1))
		{
			$hasError = true;
		}
		
		if ($hasError)
		{
			$app->enqueueMessage('Please go to Plugin Manager and enable "System - VP Email Manager" plugins.');
			
			return false;
		}
		
		return true;
	}

	/**
	* Private method to get the installed VP Email Manager plugin IDs.
	* 
	* @return mixed(boolean/array)  false if not found.
	*/
	private function getPluginID()
	{
		$db    = Factory::getDBO();
		$query = $db->getQuery(true);
		
		$query->select($db->quoteName('extension_id'))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
			->where($db->quoteName('element') . ' = ' . $db->quote('vpem'))
			->where($db->quoteName('folder') . ' = ' . $db->quote('system'));
			
		try
		{
			$db->setQuery($query);
			$plugin_id = $db->loadResult();
		}
		catch(Exception $e)
		{
			// We can not throw SQL error here.
			return false;
		}
		
		return (int) $plugin_id;
	}
	
	private function updateUserAndDateColumns()
	{
		$user  = Factory::getUser();
		$date  = Factory::getDate();
		$db    = Factory::getDBO();
		$query = $db->getQuery(true);
		
		$fields = [
			$db->quoteName('created_by') . ' = ' . $db->quote($user->get('id')),
			$db->quoteName('created_on') . ' = ' . $db->quote($date->toSql()),
			$db->quoteName('modified_by') . ' = 0',
			$db->quoteName('created_on') . ' = ' . $db->quote('0000-00-00 00:00:00')
		];
		
		$event_fields = [
			$db->quoteName('mail_from') . ' = ' . $db->quote(''),
			$db->quoteName('mail_from_name') . ' = ' . $db->quote(''),
		];
		
		$event_fields = array_merge($fields, $event_fields);
		
		$query->update($db->quoteName('#__vpem_events'))
			->set($event_fields);
		
		$db->setQuery($query);
		
		$db->execute();
		
		$query->clear()
			->update($db->quoteName('#__vpem_templates'))
			->set($fields);
		
		$db->setQuery($query);
		
		$db->execute();
	}
	
	private function deleteOutdatedFolders()
	{
		$folders = [];
		
		foreach ($folders as $folder)
		{
			if (Folder::exists($folder))
			{
				Folder::delete($folder);
			}
		}
	}
}
