<?php

/**
 * @package    VP Email Manager
 * @copyright  Copyright (C) 2012-2024 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 3 or later; see LICENSE.txt
 * @author     Abhishek Das <info@virtueplanet.com>
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Path;
use Joomla\CMS\Uri\Uri;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * VP Email Manager System Plugin
 */
class PlgSystemVpem extends JPlugin
{
    /**
     * @var    \Joomla\CMS\Application\CMSApplication
     */
    protected $app;

    /**
     * @var    boolean
     */
    private $runCron = false;

    /**
     * Constructor.
     *
     * @param   object  &$subject  The object to observe
     * @param   array   $config    An optional associative array of configuration settings.
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

        // We still need to support old VirtueMart versions.
        // We can removed this in a future version.
        defined('DS') or define('DS', DIRECTORY_SEPARATOR);

        // Register VmConfig class.
        JLoader::register('VmConfig', JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/config.php');

        if ($this->renameLoadClass('shopFunctionsF', JPATH_SITE . '/components/com_virtuemart/helpers/shopfunctionsf.php', __DIR__ . '/helpers')) {
            // Register our shopFunctionsF class
            JLoader::register('shopFunctionsF', __DIR__ . '/helpers/shopfunctionsf.php');

            // Force load shopFunctionsF class
            JLoader::load('shopFunctionsF');
        }

        JLoader::register('shopFunctionsFTest', __DIR__ . '/helpers/shopfunctionsftest.php');
    }

    /**
     * Method to load any class by renaming them
     *
     * @param  string  $className
     * @param  string  $fileName
     * @param  string  $destinationFolder
     * @param  string  $suffix
     *
     * @return  boolean  False if not loaded.
     */
    private function renameLoadClass($className, $fileName, $destinationFolder, $suffix = 'Original')
    {
        JLoader::import('joomla.filesystem.folder');
        JLoader::import('joomla.filesystem.file');

        if (class_exists($className . $suffix)) {
            return true;
        }

        if (!is_file($fileName)) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an error. ' . $this->getRelativePath($fileName) . ' is not found.');

            return false;
        }

        $parts = pathinfo($fileName);

        if (empty($parts['filename']) || empty($parts['extension'])) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an error. Invalid file ' . $this->getRelativePath($fileName));

            return false;
        }

        $copiedFileName = $destinationFolder . DIRECTORY_SEPARATOR . $parts['filename'] . $suffix . '.' . $parts['extension'];
        $hashFileName   = $destinationFolder . DIRECTORY_SEPARATOR . $parts['filename'] . $suffix . '.hash';
        $currentHash    = sha1_file($fileName);

        if (is_file($hashFileName) && is_file($copiedFileName)) {
            $oldHash = file_get_contents($hashFileName);

            if (!empty($oldHash) && $oldHash == $currentHash) {
                return require_once($copiedFileName);
            }
        }

        if (!$currentContent = file_get_contents($fileName)) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an issue. ' . $this->getRelativePath($fileName) . ' is not readable.');

            return false;
        }

        $find          = 'class ' . $className;
        $startPosition = stripos($currentContent, $find);

        if ($startPosition === false) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an issue. ' . $this->getRelativePath($fileName) . ' is not overridable.');

            return false;
        }

        $replace        = $find . $suffix;
        $currentContent = substr_replace($currentContent, $replace, $startPosition, strlen($find));

        if (!File::write($copiedFileName, $currentContent)) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an issue. ' . $this->getRelativePath($copiedFileName) . ' is not writable.');

            return false;
        }

        if (!File::write($hashFileName, $currentHash)) {
            $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . ' found an issue. ' . $this->getRelativePath($hashFileName) . ' is not writable.');

            return false;
        }

        return require_once($copiedFileName);
    }

    /**
     * After dispatch.
     *
     * @return  void
     */
    public function onAfterDispatch()
    {
        $componentEnabled = JComponentHelper::getComponent('com_vpem', true)->enabled || JComponentHelper::getComponent('vpem', true)->enabled;

        if (!$componentEnabled) {
            return;
        }

        $checkForCron = $this->isHTML();

        if (!$this->app->isClient('site') && $checkForCron) {
            $option = $this->app->input->getCmd('option');
            $view   = $this->app->input->getCmd('view');

            // Don't run cron job when on event or template editing page
            if ($option == 'com_vpem' && ($view == 'event' || $view == 'template')) {
                $checkForCron = false;
            }
        }

        if ($checkForCron) {
            $config = JComponentHelper::getParams('com_vpem');

            if ($config->get('cronjob', 1)) {
                $date  = Factory::getDate();
                $db    = Factory::getDbo();
                $query = $db->getQuery(true);

                $query->select($db->quoteName(['id', 'type', 'time']))
                    ->from($db->quoteName('#__vpem_datetime'))
                    ->where($db->quoteName('type') . ' = ' . $db->quote('last:cron'));

                $db->setQuery($query);
                $cronTime = $db->loadObject();

                $dataUpdated = false;

                if (empty($cronTime) || !is_object($cronTime)) {
                    $this->runCron = true;

                    $cronTime = new stdClass();

                    $cronTime->id   = 0;
                    $cronTime->type = 'last:cron';
                    $cronTime->time = $date->toSql();

                    $dataUpdated = true;
                } else {
                    $cronjobInterval = (int) $config->get('cronjob_interval', 60);
                    $lastCronTime    = Factory::getDate($cronTime->time);

                    $lastCronTime->modify('+' . $cronjobInterval . ' minutes');

                    if ($date >= $lastCronTime) {
                        $this->runCron = true;

                        $cronTime->time = $date->toSql();

                        $dataUpdated = true;
                    }
                }

                if ($dataUpdated) {
                    if ($cronTime->id == 0) {
                        $db->insertObject('#__vpem_datetime', $cronTime, 'id');
                    } else {
                        $db->updateObject('#__vpem_datetime', $cronTime, array('id'), false);
                    }
                }

                if ($this->runCron) {
                    Factory::getDocument()->addStyleDeclaration('.vpem-sr-only{border:0;clip:rect(0,0,0,0);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px;}');
                }
            }
        }
    }

    /**
     * After render.
     *
     * @return  void
     */
    public function onAfterRender()
    {
        // If we need to run the cron job
        if (!$this->runCron) {
            return;
        }

        // Intiate some variables
        $doc     = Factory::getDocument();
        $uri     = Uri::getInstance();
        $tab     = $doc->_getTab();
        $lineEnd = $doc->_getLineEnd();

        // Build cron job url
        $url  = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
        $url .= Uri::root(true) . '/';
        $url .= 'index.php?option=com_vpem&task=cron.run&_' . time();

        // Build html to initiate cron job
        $cron_html  = $tab . $tab . '<!-- Initiate Email Manager for VirtueMart\'s Cron Job -->' . $lineEnd;
        $cron_html .= $tab . $tab . '<img src="' . htmlspecialchars($url, ENT_COMPAT, 'UTF-8', false) . '" width="1" height="1" alt="Email Manager for VirtueMart - Cron Job" class="vpem-sr-only" aria-hidden="true" />' . $lineEnd;

        // Get body buffer
        $buffer = $this->app->getBody();

        // Add cron html just before the body closing tag
        $buffer = str_replace('</body>', $cron_html . '</body>', $buffer);

        // Set body buffer
        $this->app->setBody($buffer);
    }

    /**
     * On plugin's ajax call.
     *
     * @return json output
     */
    public function onAjaxVpem()
    {
        $user = Factory::getUser();

        if (!$this->app->isClient('site') && !$user->get('guest')) {
            // Register dlk helper class
            JLoader::register('VPComDlkHelper', JPATH_ADMINISTRATOR . '/components/com_vpem/models/fields/vpcomdlk/helper.php');

            // Get an instance of dlk helper
            $dlkHelper = VPComDlkHelper::getInstance();

            if ($this->app->input->getCmd('method') == 'validatedlk') {
                // Return validation result
                return $dlkHelper->validate();
            } elseif ($this->app->input->getCmd('method') == 'revalidatedlk') {
                // Return validation result
                return $dlkHelper->revalidate();
            } elseif ($this->app->input->getCmd('method') == 'clear') {
                // Return validation result
                return $dlkHelper->clear();
            }
        }
    }

    /**
     * Called on live update installation.
     *
     * @return  boolean
     */
    public function onInstallerBeforePackageDownload(&$url, &$headers)
    {
        // Register dlk helper class
        JLoader::register('VPComDlkHelper', JPATH_ADMINISTRATOR . '/components/com_vpem/models/fields/vpcomdlk/helper.php');

        return VPComDlkHelper::getInstance()->addDlk($url, $headers);
    }

    /**
     * Method to retrieve relative file path from absolute path
     *
     * @param string $fileName
     *
     * @return string
     */
    private function getRelativePath($fileName)
    {
        $fileName = Path::clean($fileName);
        $fileName = str_replace(JPATH_ROOT . DIRECTORY_SEPARATOR, '', $fileName);

        return $fileName;
    }

    /**
     * If the document is HTML
     *
     * @return  boolean  True if HTML document
     */
    private function isHTML()
    {
        $doc   = Factory::getDocument();
        $input = $this->app->input;

        $isFeed = ($doc->getType() == 'feed' || $input->getWord('format') == 'feed' || $input->getWord('type') == 'rss' || $input->getWord('type') == 'atom');
        $isHTML = ($doc->getType() == 'html' && !$isFeed);

        return $isHTML;
    }
}
